package org.colinvella.fancyfish.block;

import java.util.Random;

import org.colinvella.fancyfish.FancyFishMod;
import org.colinvella.fancyfish.creativetab.ModCreativeTabs;
import org.colinvella.fancyfish.item.ModItemRegistry;

import com.google.common.base.Predicate;

import net.minecraft.block.Block;
import net.minecraft.block.material.Material;
import net.minecraft.block.properties.IProperty;
import net.minecraft.block.properties.PropertyBool;
import net.minecraft.block.properties.PropertyDirection;
import net.minecraft.block.properties.PropertyInteger;
import net.minecraft.block.state.BlockState;
import net.minecraft.block.state.IBlockState;
import net.minecraft.entity.EntityLivingBase;
import net.minecraft.entity.item.EntityItem;
import net.minecraft.entity.item.EntityXPOrb;
import net.minecraft.entity.player.EntityPlayer;
import net.minecraft.init.Blocks;
import net.minecraft.item.ItemStack;
import net.minecraft.util.AxisAlignedBB;
import net.minecraft.util.BlockPos;
import net.minecraft.util.EnumFacing;
import net.minecraft.util.EnumParticleTypes;
import net.minecraft.util.EnumWorldBlockLayer;
import net.minecraft.world.IBlockAccess;
import net.minecraft.world.World;
import net.minecraftforge.fml.relauncher.Side;
import net.minecraftforge.fml.relauncher.SideOnly;

public class ClamBlock extends ModBlock
{
	public static final String ID = "Clam";
	
	// properties mapped to meta data
    public static final PropertyInteger PROPERTY_LEVEL = PropertyInteger.create("level", 0, 15);
	public static final PropertyBool PROPERTY_OPEN = PropertyBool.create("open");
    public static final PropertyDirection PROPERTY_FACING = PropertyDirection.create("facing",
    	new Predicate()
	    {
	        public boolean apply(EnumFacing facing)
	        {
	            return facing != EnumFacing.DOWN && facing != EnumFacing.UP ;
	        }
	        public boolean apply(Object facing)
	        {
	            return this.apply((EnumFacing)facing);
	        }
	    });	  
    
	public ClamBlock()
	{
		super(ID, Material.water, ModCreativeTabs.FancyFishTab);
		this.setLightOpacity(0); // light passes through
		this.setTickRandomly(true);
		
		// bounding box for closed shell
		this.minX = this.minZ = 0.125;
		this.minY = 0;
		this.maxX = this.maxZ = 0.875;
		this.maxY = 0.25;
	}

	// override: return false as it is an irregular block
	@Override
	public boolean isOpaqueCube()
	{
		return false;
	}
	
	// bounding box depends on clam open or closed
	@Override
	public AxisAlignedBB getCollisionBoundingBox(World world, BlockPos pos, IBlockState state)
	{
		// default is for closed clam
		AxisAlignedBB axisAlignedBB = super.getCollisionBoundingBox(world, pos, state);
		
		// expand AABB in Y direction for open clam
		if ((Boolean)state.getValue(PROPERTY_OPEN))
			return axisAlignedBB.expand(0, 0.25, 0);
		else
			return axisAlignedBB;
	};
	
	// override: treat block as translucent due to light shade
	@Override
	public EnumWorldBlockLayer getBlockLayer()
	{
		return EnumWorldBlockLayer.TRANSLUCENT;
	}

	@Override
	public IBlockState onBlockPlaced(World world, BlockPos pos, EnumFacing facing
			, float hitX, float hitY, float hitZ, int meta, EntityLivingBase placer)
	{
		// placed clam is open by default, unless it is in creative mode
		boolean clamOpen = true;
		if (placer instanceof EntityPlayer)
		{
			clamOpen = !((EntityPlayer)placer).capabilities.isCreativeMode;
		}
		
		IBlockState clamBlockState = this.getStateFromMeta(meta)
			.withProperty(PROPERTY_OPEN, clamOpen)
			.withProperty(PROPERTY_FACING, placer.getHorizontalFacing());
		return clamBlockState;
	};
	
	// open clam if closed
	@Override 
	public boolean onBlockActivated(World world, BlockPos pos, IBlockState blockState, EntityPlayer player,
		EnumFacing side, float hitX, float hitY, float hitZ)
	{
		if (world.isRemote)
			return true;
		
		IBlockState clamClosedState = blockState.withProperty(PROPERTY_OPEN, false);
		IBlockState clamOpenState = blockState.withProperty(PROPERTY_OPEN, true);
		
		if (blockState == clamOpenState)
			return true;

		world.setBlockState(pos, clamOpenState);
		world.markBlockForUpdate(pos);
		
		double midX = pos.getX() + 0.5;
		double midY = pos.getY() + 0.5;
		double midZ = pos.getZ() + 0.5;
        world.playSoundEffect(midX, midY, midZ, CLAM_OPEN, 1F, 1F);
		
        // 20% chance of spawning pearl and some xp
        if (this.RANDOM.nextInt(5) > 0)
        	return true;
        
        world.spawnEntityInWorld(new EntityItem(world, midX, midY, midZ, new ItemStack(ModItemRegistry.Pearl)));
        world.playSoundEffect(midX, midY, midZ, "random.levelup", 1.0F, 0.8f);
        
		int exp = 10;
		while (exp > 0)
		{
			int expFragment = EntityXPOrb.getXPSplit(exp);
			exp -= expFragment;
			world.spawnEntityInWorld(new EntityXPOrb(world, midX, midY, midZ, expFragment));
		}
		
		return true;
	};
	
    // whether this Block is solid on the given Side
    @Override
    public boolean isBlockSolid(IBlockAccess world, BlockPos pos, EnumFacing side)
    {
        Material material = world.getBlockState(pos).getBlock().getMaterial();
        
		if (material == this.blockMaterial || material == Material.water)
        	return false;

        if (side == EnumFacing.UP)
        	return true;
        
        if (material == Material.ice)
        	return false;
        
        return super.isBlockSolid(world, pos, side);
    }
	
	@Override
    @SideOnly(Side.CLIENT)
    public boolean shouldSideBeRendered(IBlockAccess worldIn, BlockPos pos, EnumFacing side)
    {
		Material material = worldIn.getBlockState(pos).getBlock().getMaterial();
		
		if (material == this.blockMaterial || material == Material.water)
			return false;
		
        if (side == EnumFacing.UP)
        	return true;

        return super.shouldSideBeRendered(worldIn, pos, side);
    }
	
	@Override
	public boolean canPlaceBlockAt(World world, BlockPos pos)
	{
		// target block must be water
		Block block = world.getBlockState(pos).getBlock();
		if (block != Blocks.water)
			return false;

		// block on which clams lie must be dirt, sand or clay
		if (pos.getY() < 1)
			return false;

		Block blockBelow = world.getBlockState(pos.offset(EnumFacing.DOWN)).getBlock();
		if (!canPlaceClamOn(blockBelow))
			return false;

		// ensure there is no air or waterlilly above to limit clam to one block away from water surface
		if (pos.getY() >= world.getHeight() - 1)
			return false;

		Block blockAbove = world.getBlockState(pos.offset(EnumFacing.UP)).getBlock();
		return blockAbove != Blocks.air && blockAbove != Blocks.waterlily;
	};
	
	// break clam if not surrounded by still water or supporting solid block
	@Override public void onNeighborBlockChange(World world, BlockPos pos, IBlockState blockState, Block neighborBlock)
	{
		if (world.isRemote)
			return;

		BlockPos posAbove = pos.offset(EnumFacing.UP);
		Block blockAbove = world.getBlockState(posAbove).getBlock();

		// if exposed to surface or waterlilly, break it
		if (blockAbove == Blocks.air || blockAbove == Blocks.waterlily)
		{
			breakClam(world, pos, true);
		}

		// if block below is no longer good ground, then break this block
		BlockPos posBelow = pos.offset(EnumFacing.DOWN);
		Block blockBelow = world.getBlockState(posBelow).getBlock();
		if (!canPlaceClamOn(blockBelow))
		{
			breakClam(world, pos, true);
		}
	}
	
	@Override
	public void breakBlock(World world, BlockPos pos, IBlockState blockState)
	{
		//breakClam(world, pos, false);
		super.breakBlock(world, pos, blockState);
	};
	
	@Override
	public void randomDisplayTick(World world, BlockPos pos, IBlockState state, Random random)
	{
		// run 25% of the time
		if (random.nextInt(4) > 0)
			return;
		
		// this runs only on client
		double bubbleX = pos.getX() + 0.25 + random.nextDouble() * 0.5;
		double bubbleY = pos.getY() + 0.25;
		double bubbleZ = pos.getZ() + 0.25 + random.nextDouble() * 0.5;
		world.spawnParticle(EnumParticleTypes.WATER_BUBBLE, bubbleX, bubbleY, bubbleZ, 0, 1, 0);
	};
	
		
	// getStateFromMeta, getMetaFromState are used to inter-convert between the block's property values and
	//   the stored metadata (which must be an integer in the range 0 - 15 inclusive)
	@Override
	public IBlockState getStateFromMeta(int meta)
	{
		boolean isOpen = meta >= 4;
		int facingBits = meta % 4;
		EnumFacing facing = EnumFacing.getHorizontal(facingBits);
		return this.getDefaultState().withProperty(PROPERTY_OPEN, isOpen).withProperty(PROPERTY_FACING, facing);
	}
	
	@Override
	public int getMetaFromState(IBlockState state)
	{
		boolean isOpen = (Boolean)state.getValue(PROPERTY_OPEN);	
		EnumFacing facing = (EnumFacing)state.getValue(PROPERTY_FACING);
		int facingBits = facing.getIndex() - 2;
		return isOpen ? 4 + facingBits : facingBits;
	}
	
	// necessary to define which properties your blocks use
	// will also affect the variants listed in the block states model file
	@Override
	protected BlockState createBlockState()
	{
	    return new BlockState(this, new IProperty[] {PROPERTY_LEVEL, PROPERTY_OPEN, PROPERTY_FACING});
	}
		
	private boolean canPlaceClamOn(Block block)
	{
		return block == Blocks.sand || block == Blocks.dirt || block == Blocks.clay;
	}
	
	private void breakClam(World world, BlockPos pos, boolean spawnClamItem)
	{
		Block block = world.getBlockState(pos).getBlock();
		
		// replace clam with still water
		if (block == Blocks.air)
			world.setBlockState(pos, Blocks.water.getDefaultState().withProperty(PROPERTY_LEVEL, 15));
		
		double midX = pos.getX() + 0.5;
		double midY = pos.getY() + 0.5;
		double midZ = pos.getZ() + 0.5;
		
		// spawn clam as an item if requested
		if (spawnClamItem)
		{
			//world.spawnEntityInWorld(
			//	new EntityItem(world, midX, midY, midZ, new ItemStack(ModBlockRegistry.Kelp)));
		}
	}
	
	private static final String CLAM_OPEN = FancyFishMod.MOD_ID + ":block.clam.Open";
}
